@@ -1,46 +1,18 @@
-=encoding UTF-8
-
-=head1 NAME
-
-Lingua::JA::Moji - Handle many kinds of Japanese characters
-
-=head1 SYNOPSIS
-
-Convert various types of characters into one another.
-
-    use Lingua::JA::Moji qw/kana2romaji romaji2kana/;
-    use utf8;
-    my $romaji = kana2romaji ('あいうえお');
-    # $romaji is now 'aiueo'.
-    my $kana = romaji2kana ($romaji);
-    # $kana is now 'アイウエオ'.
-
-=head1 EXPORT
-
-This module does not export any functions except on request.
-
-=head1 ENCODING
-
-All the functions in this module assume that you are using Perl's
-Unicode encoding, and all input and output strings must be encoded
-using Perl's so-called "utf8".
-
-=head1 FUNCTIONS
-
-=cut
-
 package Lingua::JA::Moji;
 
 use warnings;
 use strict;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
+
 use Carp;
-use Lingua::JA::Moji::Convertor qw/load_convertor make_convertors/;
-use Convert::Moji;
+#use Lingua::JA::Moji::Convertor qw/load_convertor make_convertors/;
+use Convert::Moji qw/make_regex length_one unambiguous/;
 use utf8;
 use File::ShareDir 'dist_file';
+
 require Exporter;
+
 our @ISA = qw(Exporter);
 
 our @EXPORT_OK = qw/
@@ -67,9 +39,166 @@ our @EXPORT_OK = qw/
                     kana2circled
                     circled2kana
                     normalize_romaji
+                    new2old_kanji
+                    old2new_kanji
                     /;
 
-our $AUTOLOAD;
+our %EXPORT_TAGS = (
+    'all' => \@EXPORT_OK,
+);
+
+# Load a specified convertor from the shared directory.
+
+sub load_convertor
+{
+    my ($in, $out) = @_;
+    my $filename = $in."2".$out.'.txt';
+    my $file = dist_file ('Lingua-JA-Moji', $filename);
+    if (! $file || ! -f $file) {
+	croak "Could not find distribution file '$filename'";
+    }
+    return Convert::Moji::load_convertor ($file);
+}
+
+sub add_boilerplate
+{
+    my ($code, $name) = @_;
+    $code =<<EOSUB;
+sub convert_$name
+{
+    my (\$conv,\$input,\$convert_type) = \@_;
+    $code
+    return \$input;
+}
+EOSUB
+$code .= "\\\&".__PACKAGE__."::convert_$name;";
+#    print $code,"\n";
+    return $code;
+}
+
+sub ambiguous_reverse
+{
+    my ($table) = @_;
+    my %inverted;
+    for (keys %$table) {
+	my $val = $table->{$_};
+#	print "Valu is $val\n";
+	push @{$inverted{$val}}, $_;
+#	print "key $_ stuff ",join (' ',@{$inverted{$val}}),"\n";
+    }
+    return \%inverted;
+}
+
+# Callback
+
+sub split_match
+{
+    my ($conv, $input, $convert_type) = @_;
+    $convert_type = "all" if (!$convert_type);
+#    print "Convert type is '$convert_type'\n";
+    my @input = split '', $input;
+    my @output;
+    for (@input) {
+	my $in = $conv->{out2in}->{$_};
+#	print "$_ $in\n";
+	# No conversion defined.
+	if (! $in) {
+	    push @output, $_;
+	    next;
+	}
+	# Unambigous case
+	if (@{$in} == 1) {
+	    push @output, $in->[0];
+	    next;
+	}
+	if ($convert_type eq 'all') {
+	    push @output, $in;
+	} elsif ($convert_type eq 'first') {
+	    push @output, $in->[0];
+	} elsif ($convert_type eq 'random') {
+	    my $pos = int rand @$in;
+#	    print "RANDOM $pos\n";
+	    push @output, $in->[$pos];
+	}
+    }
+    return \@output;
+}
+
+sub make_convertors
+{
+    my $conv = {};
+    my ($in, $out, $table) = @_;
+    if (!$table) {
+	$table = load_convertor ($in, $out);
+    }
+    $conv->{in2out} = $table;
+    my @keys = keys %{$table};
+    my @values = values %{$table};
+    my $sub_in2out;
+    my $sub_out2in;
+    if (length_one(@keys)) {
+	my $lhs = join '', @keys;
+
+	# Improvement: one way tr/// for the ambiguous case lhs/rhs only.
+
+	if (length_one(@values) && unambiguous($table)) {
+#	    print "Not ambiguous\n";
+	    # can use tr///;
+	    my $rhs = join '', @values;
+	    $sub_in2out = "\$input =~ tr/$lhs/$rhs/;";
+	    $sub_out2in = "\$input =~ tr/$rhs/$lhs/;";
+	}
+        else {
+	    $sub_in2out = "\$input =~ s/([$lhs])/\$conv->{in2out}->{\$1}/eg;";
+	    my $rhs = make_regex (@values);
+	    if (unambiguous($conv->{in2out})) {
+		my %out2in_table = reverse %{$conv->{in2out}};
+		$conv->{out2in} = \%out2in_table;
+		$sub_out2in = "\$input =~ s/($rhs)/\$conv->{out2in}->{\$1}/eg;";
+	    } else {
+#		print "Unambiguous inversion is not possible with $in, $out.\n";
+		$conv->{out2in} = ambiguous_reverse ($conv->{in2out});
+		$sub_out2in = "\$input = \$conv->split_match (\$input, \$convert_type);";
+	    }
+	}
+    }
+    else {
+	my $lhs = make_regex (@keys);
+	$sub_in2out = "\$input =~ s/($lhs)/\$conv->{in2out}->{\$1}/eg;";
+	my $rhs = make_regex (@values);
+	if (unambiguous($conv->{in2out})) {
+	    my %out2in_table = reverse %{$conv->{in2out}};
+	    $conv->{out2in} = \%out2in_table;
+	    $sub_out2in = "    \$input =~ s/($rhs)/\$conv->{out2in}->{\$1}/eg;";
+	}
+    }
+    $sub_in2out = add_boilerplate ($sub_in2out, "${in}2$out");
+    my $sub1 = eval $sub_in2out;
+    $conv->{in2out_sub} = $sub1;
+    if ($sub_out2in) {
+	$sub_out2in = add_boilerplate ($sub_out2in, "${out}2$in");
+	my $sub2 = eval $sub_out2in;
+	if ($@) {
+	    print "Errors are ",$@,"\n";
+	    print "\$sub2 = ",$sub2,"\n";
+	}
+	$conv->{out2in_sub} = $sub2;
+    }
+    bless $conv;
+    return $conv;
+}
+
+sub convert
+{
+    my ($conv, $input) = @_;
+    return &{$conv->{in2out_sub}}($conv, $input);
+}
+
+sub invert
+{
+    my ($conv, $input, $convert_type) = @_;
+    return &{$conv->{out2in_sub}}($conv, $input, $convert_type);
+}
 
 
 # Kana ordered by consonant. Adds bogus "q" gyou for small vowels and
@@ -134,10 +263,6 @@ for my $vowel (keys %段) {
     $vowelclass{$vowel} = join '', @kana_list;
 }
 
-#for my $kana (sort keys %子音) {
-#    print "$kana: ",$子音{$kana},$母音{$kana},"\n";
-#}
-
 # Kana gyou which can be preceded by a sokuon (small tsu).
 
 # Added d to the list for ウッド BKB 2010-07-20 23:27:07
@@ -209,52 +334,6 @@ my %長音表記;
 @{$長音表記{passport}}{@あいうえお}   = qw/a  i  u  e  oh oh/;
 @{$長音表記{none}}{@あいうえお}       = qw/a  i  u  e  o  o/;
 
-=head2 kana2romaji
-
-    use utf8;
-    my $romaji = kana2romaji ("うれしいこども");
-
-    # $romaji = "uresiikodomo"
-
-Convert kana to a romanized form.
-
-An optional second argument, a hash reference, controls the style of
-conversion.
-
-    use utf8;
-    my $romaji = kana2romaji ("しんぶん", {style => "hepburn"});
-    # $romaji = "shimbun"
-
-The possible options are
-
-=over
-
-=item style
-
-The style of romanization. The default form of romanization is
-"Nippon-shiki". See
-L<http://www.sljfaq.org/afaq/nippon-shiki.html>. The user can set the
-conversion style to "hepburn" or "passport" or "kunrei". See
-L<http://www.sljfaq.org/afaq/kana-roman.html>.
-
-=item use_m
-
-If this is set to any "true" value, syllabic I<n>s (ん) which come
-before "b" or "p" sounds, such as the first "n" in "shinbun" (しんぶん,
-newspaper) will be converted into "m" rather than "n".
-
-=item ve_type
-
-C<ve_type> controls how long vowels are written. The default is to use
-circumflexes to represent long vowels. If you set "ve_type" =>
-"macron", then it uses macrons (the Hepburn system). If you set
-C<< "ve_type" => "passport" >>, then it uses "oh" to write long "o"
-vowels. If you set C<< "ve_type" => "none" >>, then it does not use "h".
-
-=back
-
-=cut
-
 sub kana2romaji
 {
     # Parse the options
@@ -281,7 +360,6 @@ sub kana2romaji
     }
     my $wapuro;
     $wapuro   = 1 if $options->{wapuro};
-#    print "wapuro is $wapuro\n";
     my $use_m = 0;
     if ($hepburn || $passport) { $use_m = 1 }
     if (defined($options->{use_m})) { $use_m = $options->{use_m} }
@@ -290,7 +368,6 @@ sub kana2romaji
 	$ve_type = 'macron';
     }
     if ($wapuro) {
-#        print "Wapuro romanization\n";
         $ve_type = 'wapuro';
     }
     if ($passport) {
@@ -383,17 +460,6 @@ sub kana2romaji
     return $input;
 }
 
-=head2 romaji2hiragana
-
-    my $hiragana = romaji2hiragana ('babubo');
-
-Convert romanized Japanese into hiragana. This takes the same options
-as L<romaji2kana>. It also switches on the "wapuro" option which makes
-the use of long vowels with a kana rather than a chouon (long vowel
-marker).
-
-=cut
-
 sub romaji2hiragana
 {
     my ($input, $options) = @_;
@@ -405,23 +471,6 @@ sub romaji2hiragana
     return kata2hira ($katakana);
 }
 
-
-=head2 romaji_styles
-
-    my @styles = romaji_styles ();
-    # Returns a true value
-    romaji_styles ("hepburn");
-    # Returns the undefined value
-    romaji_styles ("frogs");
-
-Given an argument, return whether it is a legitimate style of romanization.
-
-Without an argument, return a list of possible styles, as an array of
-hash values, with each hash element containing "abbrev" as a short
-name and "full_name" for the full name of the style.
-
-=cut
-
 sub romaji_styles
 {
     my ($check) = @_;
@@ -500,31 +549,11 @@ my %longvowels;
 @longvowels{qw/ā  ī  ū  ē  ō/}  = qw/aー iー uー eー oー/;
 my $longvowels = join '|', sort {length($a)<=>length($b)} keys %longvowels;
 
-=head2 romaji2kana
-
-     my $kana = romaji2kana ('yamaguti');
-     # $kana = 'ヤマグチ';
-
-
-Convert romanized Japanese to kana. The romanization is highly liberal
-and will attempt to convert any romanization it sees into kana.
-
-     my $kana = romaji2kana ($romaji, {wapuro => 1});
-
-Use an option C<< wapuro => 1 >> to convert long vowels into the
-equivalent kana rather than I<chouon>.
-
-Convert romanized Japanese (romaji) into katakana. If you want to
-convert romanized Japanese into hiragana, use L<romaji2hiragana>
-instead of this.
-
-=cut
-
 sub romaji2kana
 {
-    if (!$romaji2katakana) {
+    if (! defined $romaji2katakana) {
 	$romaji2katakana = load_convertor ('romaji','katakana');
-	$romaji_regex = Lingua::JA::Moji::Convertor::make_regex (keys %$romaji2katakana);
+	$romaji_regex = make_regex (keys %$romaji2katakana);
     }
     my ($input, $options) = @_;
     $input = lc $input;
@@ -549,23 +578,10 @@ sub romaji2kana
     # oh{consonant} -> oo
     $input =~ s/oh(?=[ksthmrgzdbp])/オオ/g;
     # Substitute all the kana.
-    $input =~ s/($romaji_regex)/$$romaji2katakana{$1}/g;
-    #print "input = $input\n";
+    $input =~ s/($romaji_regex)/$romaji2katakana->{$1}/g;
     return $input;
 }
 
-=head2 is_voiced
-
-    if (is_voiced ('が')) {
-         print "が is voiced.\n";
-    }
-
-Given a kana or romaji input, C<is_voiced> returns a true value if the
-sound is a voiced sound like I<a>, I<za>, I<ga>, etc. and the
-undefined value if not.
-
-=cut
-
 sub is_voiced
 {
     my ($sound) = @_;
@@ -584,23 +600,6 @@ sub is_voiced
     }
 }
 
-=head2 is_romaji
-
-    # The following line returns "undef"
-    is_romaji ("abcdefg");
-    # The following line returns a defined value
-    is_romaji ("atarimae");
-
-Detect whether a string of alphabetical characters, which may also
-include characters with macrons or circumflexes, "looks like"
-romanized Japanese. If the test is successful, returns the romaji in a
-canonical form.
-
-This functions by converting the string to kana and seeing if it
-converts cleanly or not.
-
-=cut
-
 sub is_romaji
 {
     my ($romaji) = @_;
@@ -616,21 +615,6 @@ sub is_romaji
     return;
 }
 
-=head2 hira2kata
-
-    my $katakana = hira2kata ($hiragana);
-
-C<hira2kata> converts hiragana into katakana. If the input is a list,
-it converts each element of the list, and if required, returns a list
-of the converted inputs, otherwise it returns a concatenation of the
-strings.
-
-    my @katakana = hira2kata (@hiragana);
-
-This does not convert chouon signs.
-
-=cut
-
 sub hira2kata
 {
     my (@input) = @_;
@@ -638,23 +622,6 @@ sub hira2kata
     return wantarray ? @input : "@input";
 }
 
-=head2 kata2hira
-
-     my $hiragana = kata2hira ('カキクケコ');
-     # $hiragana = 'かきくけこ';
-
-C<kata2hira> converts full-width katakana into hiragana. If the input
-is a list, it converts each element of the list, and if required,
-returns a list of the converted inputs, otherwise it returns a
-concatenation of the strings.
-
-    my @hiragana = hira2kata (@katakana);
-
-This function does not convert chouon signs into long vowels. It also
-does not convert half-width katakana into hiragana.
-
-=cut
-
 sub kata2hira
 {
     my (@input) = @_;
@@ -685,7 +652,7 @@ sub load_strip_daku
 	    map {$_."゛"} (make_dak_list (qw/k t s h/));
 	@濁点{(make_dak_list ('p'))} = map {$_."゜"} (make_dak_list ('h'));
 	my $濁点 = join '', keys %濁点;
-	$strip_daku = make_convertors("ten_joined", "ten_split", \%濁点);
+	$strip_daku = make_convertors ("ten_joined", "ten_split", \%濁点);
     }
 }
 
@@ -705,79 +672,31 @@ sub kana2hw2
 {
     my $conv = Convert::Moji->new (["oneway", "tr", "あ-ん", "ア-ン"],
 				   ["file",
-				    getdistfile("katakana2hw_katakana")]);
+				    getdistfile ("katakana2hw_katakana")]);
     return $conv;
 }
 
 my $kata2hw;
 
-=head2 kana2hw
-
-     my $half_width = kana2hw ('あいウカキぎょう。');
-     # $half_width = 'ｱｲｳｶｷｷﾞｮｳ｡'
-
-C<kana2hw> converts hiragana, katakana, and fullwidth Japanese
-punctuation to halfwidth katakana and halfwidth punctuation. Its
-function is similar to the Emacs command C<japanese-hankaku-region>.
-For the opposite function,
-see L<hw2katakana>.
-
-=cut
-
 sub kana2hw
 {
    my ($input) = @_;
    $input = hira2kata ($input);
    if (!$kata2hw) {
        $kata2hw = make_convertors ('katakana','hw_katakana');
-#       print $kata2hw->{カ},"\n";
    }
-#   $input =~ tr/あ-ん/ア-ン/;
-#   while ($input =~ /([ア-ン])/g) {
-#       print ".";
-#       print $$kata2hw{$1};
-#   }
-#   $input =~ s/([ア-ン])/$$kata2hw{$1}/g;
-   
    return $kata2hw->convert ($input);
 }
 
-=head2 hw2katakana
-
-     my $full_width = hw2katakana ('ｱｲｳｶｷｷﾞｮｳ｡');
-     # $full_width = 'アイウカキギョウ。'；
-
-C<hw2katakana> converts halfwidth katakana and Japanese punctuation to
-fullwidth katakana and punctuation. Its function is similar to the
-Emacs command C<japanese-zenkaku-region>. For the opposite function,
-see L<kana2hw>.
-
-=cut
-
 sub hw2katakana
 {
     my ($input) = @_;
    if (!$kata2hw) {
        $kata2hw = make_convertors ('katakana','hw_katakana');
-#       print $kata2hw->{カ},"\n";
    }
-#    $input =~ s/($hwregex)/${$hwtable}{$1}/g;
     return $kata2hw->invert ($input);
 }
 
-=head2 InHankakuKatakana
-
-    use Lingua::JA::Moji qw/InHankakuKatakana/;
-    use utf8;
-    if ('ｱ' =~ /\p{InHankakuKatakana}/) {
-        print "ｱ is half-width katakana\n";
-    }
-
-C<InHankakuKatakana> is a character class for use in regular
-expressions with C<\p> which can validate halfwidth katakana.
-
-=cut
-
 sub InHankakuKatakana
 {
     return <<'END';
@@ -786,16 +705,6 @@ sub InHankakuKatakana
 END
 }
 
-=head2 wide2ascii
-
-     my $ascii = wide2ascii ('ａｂＣＥ０１９');
-     # $ascii = 'abCE019'
-
-Convert the "wide ASCII" used in Japan (fullwidth ASCII, 全角英数字)
-into usual ASCII symbols (半角英数字).
-
-=cut
-
 sub wide2ascii
 {
     my ($input) = @_;
@@ -803,14 +712,6 @@ sub wide2ascii
     return $input;
 }
 
-=head2 ascii2wide
-
-Convert usual ASCII symbols (半角英数字) into the "wide ASCII" used in
-Japan (fullwidth ASCII, 全角英数字).
-
-
-=cut
-
 sub ascii2wide
 {
     my ($input) = @_;
@@ -818,19 +719,6 @@ sub ascii2wide
     return $input;
 }
 
-=head2 InWideAscii
-
-    use Lingua::JA::Moji qw/InWideAscii/;
-    use utf8;
-    if ('Ａ' =~ /\p{InWideAscii}/) {
-        print "Ａ is wide ascii\n";
-    }
-
-This is a character class for use with \p which matches a "wide ascii"
-(全角英数字).
-
-=cut
-
 sub InWideAscii
 {
     return <<'END';
@@ -844,16 +732,10 @@ my $kana2morse;
 sub load_kana2morse
 {
     if (!$kana2morse) {
-	$kana2morse = Lingua::JA::Moji::make_convertors ('katakana', 'morse');
+	$kana2morse = make_convertors ('katakana', 'morse');
     }
 }
 
-=head2 kana2morse
-
-Convert Japanese kana into Morse code
-
-=cut
-
 sub kana2morse
 {
     my ($input) = @_;
@@ -923,13 +805,6 @@ for my $k (keys %行) {
     }
 }
 
-=head2 is_kana
-
-Returns a true value if its argument is a string of kana, or an
-undefined value if not.
-
-=cut
-
 sub is_kana
 {
     my ($may_be_kana) = @_;
@@ -939,13 +814,6 @@ sub is_kana
     return;
 }
 
-=head2 is_hiragana
-
-Returns a true value if its argument is a string of kana, or an
-undefined value if not.
-
-=cut
-
 sub is_hiragana
 {
     my ($may_be_kana) = @_;
@@ -955,12 +823,6 @@ sub is_hiragana
     return;
 }
 
-=head2 kana2katakana
-
-Convert either katakana or hiragana to katakana.
-
-=cut
-
 sub kana2katakana
 {
     my ($input) = @_;
@@ -993,20 +855,14 @@ sub brailletransinv {s/([⠐⠠])(.)/$2$1/g;return $_}
 
 sub kana2braille2
 {
-    my $conv = Convert::Moji->new (["table",\%濁点],
-				   ["code",\&brailleon,\&brailleback],
-				   ["file",getdistfile ("katakana2braille")],
-				   ["code",\&brailletrans,\&brailletransinv],
+    my $conv = Convert::Moji->new (["table", \%濁点],
+				   ["code", \&brailleon,\&brailleback],
+				   ["file", getdistfile ("katakana2braille")],
+				   ["code", \&brailletrans,\&brailletransinv],
 			       );
     return $conv;
 }
 
-=head2 kana2braille
-
-Converts kana into the equivalent Japanese braille (I<tenji>) forms.
-
-=cut
-
 
 sub kana2braille
 {
@@ -1026,12 +882,6 @@ sub kana2braille
     return $input;
 }
 
-=head2 braille2kana
-
-Converts Japanese braille (I<tenji>) into the equivalent katakana.
-
-=cut
-
 sub braille2kana
 {
     my ($input) = @_;
@@ -1055,12 +905,6 @@ sub load_circled_conv
     }
 }
 
-=head2 kana2circled
-
-C<kana2circled> converts kana into the "circled katakana" of Unicode.
-
-=cut
-
 sub kana2circled
 {
     my ($input) = @_;
@@ -1072,30 +916,16 @@ sub kana2circled
     return $input;
 }
 
-=head2 circled2kana
-
-C<circled2kana> converts the "circled katakana" of Unicode into the
-usual katakana.
-
-=cut
-
 sub circled2kana
 {
     my ($input) = @_;
     load_circled_conv;
+    load_strip_daku;
     $input = $circled_conv->invert ($input);
-    $input = $strip_daku->invert($input);
+    $input = $strip_daku->invert ($input);
     return $input;
 }
 
-=head2 normalize_romaji
-
-C<normalize_romaji> converts romanized Japanese to a canonical form,
-which is based on the Nippon-shiki romanization, but without
-representing long vowels using a circumflex.
-
-=cut
-
 sub normalize_romaji
 {
     my ($romaji) = @_;
@@ -1104,119 +934,34 @@ sub normalize_romaji
     my $romaji_out = kana2romaji ($kana, {ve_type => 'wapuro'});
 }
 
-# sub AUTOLOAD {
-#     my ($input) = @_;
-#     my $name = $AUTOLOAD;
-#     $name =~ s/.*://;   # strip fully-qualified portion
-#     if ($name =~ /(\w+)2(\w+)/) {
-# 	my $incode = $1;
-# 	if ($incode eq 'kana') {
-# 	    $input = kana2katakana($input);
-# 	    $incode = 'katakana';
-# 	}
-# 	my $conv = make_convertors ($incode, $2);
-# 	if ($conv) {
-# 	    return $conv->convert($input);
-# 	}
-#     }
-#     print STDERR "Can't find a suitable convertor for '$name'.\n";
-#     return;
-# }
+my $new2old_kanji;
 
-1; # End of Lingua::JA::Moji
+sub load_new2old_kanji
+{
+    $new2old_kanji = Convert::Moji->new (
+        ['file', getdistfile ('new_kanji2old_kanji')],
+    );
+}
 
+sub new2old_kanji
+{
+    my ($new_kanji) = @_;
+    if (! $new2old_kanji) {
+        load_new2old_kanji ();
+    }
+    my $old_kanji = $new2old_kanji->convert ($new_kanji);
+    return $old_kanji;
+}
 
-__END__
+sub old2new_kanji
+{
+    my ($old_kanji) = @_;
+    if (! $new2old_kanji) {
+        load_new2old_kanji ();
+    }
+    my $new_kanji = $new2old_kanji->invert ($old_kanji);
+    return $new_kanji;
+}
 
-=head1 AUTHOR
+1; 
 
-Ben Bullock, C<< <bkb@cpan.org> >>
-
-=head1 SUPPORT
-
-=head2 Mailing list
-
-I have set up a mailing list for this module and L<Convert::Moji> at
-L<http://groups.google.com/group/perl-moji>. If you have any questions
-about either of these modules, please ask on the mailing list rather
-than sending me email, because I would prefer that a record of the
-conversation can be kept for the future reference of other users.
-
-=head2 Examples
-
-For examples of this module in use, see my website
-L<http://www.lemoda.net/lingua-ja-moji/index.html>. This page links to
-examples which I've set up on the web specifically to show this module
-in action.
-
-=head2 Bugs
-
-Please send bug reports to the Perl bug tracker at rt.cpan.org, or
-send them to the mailing list.
-
-There are some known bugs or issues with romaji to kana conversion and
-vice-versa. I'm still working on these.
-
-=head1 STATUS
-
-This module is "alpha" (that is a computerese euphemism for "the
-module is badly-formed and unfinished") and the external interface is
-liable to change drastically in the future. If you have a request,
-please speak up.
-
-Please also note that some of this documentation is not finished yet,
-some of the functions documented here don't exist yet.
-
-=head1 SEE ALSO
-
-There are some other useful Perl modules already on CPAN as follows.
-
-=head2 Japanese kana/romanization
-
-=over
-
-=item L<Data::Validate::Japanese>
-
-This is where I got several of the ideas for this module from. It
-contains validators for kanji and kana.
-
-=item L<Lingua::JA::Kana>
-
-This is where I got several of the ideas for this module from. It
-contains convertors for hiragana, katakana (fullwidth only), and
-romaji. The romaji conversion is less complete than this module but
-more compact and probably much faster, if you need high speed
-romanization.
-
-=item L<Lingua::JA::Romanize::Japanese>
-
-Romanization of Japanese. The module also includes romanization of
-kanji via the kakasi kanji to romaji convertor, and other functions.
-
-=item L<Lingua::JA::Romaji::Valid>
-
-Validate romanized Japanese.
-
-=item L<Lingua::JA::Hepburn::Passport>
-
-=back
-
-=head2 Other
-
-=over
-
-=back
-
-=head1 ACKNOWLEDGEMENTS
-
-Thanks to Naoki Tomita for various assitances (see
-L<http://groups.google.com/group/perl-moji/browse_thread/thread/10a42c35f7c22ebc>).
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2008-2010 Ben Bullock, all rights reserved.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-=cut
